#!/usr/bin/perl
# -*- mode: Perl; tab-width: 4; -*-

# Updated 11/16/2015 by Eric Smith (tweaked outputs and disabled the format function for internal compatibility with other BD tools)
# Updated 8/3/2015 by Eric Smith (made HTML output option)
# Updated 5/8/2015 by Eric Smith (made stricter Perl, some bugfixes)
# Updated 6/3/2014 by Rodolphe Egret(corrected extensions)
# Updated 5/7/2014 by Michael Nolin (corrected output order)
# Created on February 1, 2005
# Copyright (C) 2005 Black Duck Software Inc.
# http://www.blackducksoftware.com/
# All rights reserved.
#
# This software is the confidential and proprietary information of
# Black Duck Software ("Confidential Information").  You shall not
# disclose such Confidential Information and shall use it only in
# accordance with the terms of the license agreement you entered into
# with Black Duck Software.
# $Id: bdsest.pl,v 2.01 2015/11/16 10:36:00 ess Exp $;
 
# --------------------------------------------------------------------
#
# This script estimates how many megabytes are in your project, so you
# know what size scans to buy in protexIP/OnDemand.  To run it:
#
#    1. Ensure that perl is installed somewhere on your system.
#    2. Save this file as "bdsest.pl" in the current directory
#    3. Run 'perl bdsest.pl PROJECT_DIRECTORY', replacing
#       PROJECT_DIRECTORY with the appropriate path.
#
#		Note: if on a Windows system, the PROJECT_DIRECTORY
#				should have double quotes around it, particularly
#				if there is a space anywhere on that directory path.
#		e.g. C:\Program Files (x86) -> "C:\Program Files (x86)"
#
#    4. It will then return an HTML file called BD_MBE.html in the same directory where the script resides
# --------------------------------------------------------------------

use strict;
use warnings;
use File::Find;
use File::Basename;
use File::Spec;
use Cwd 'abs_path';

my @dir_names = (@ARGV > 0) ? $ARGV[0] : (".");
if (@ARGV == 0 || lc($ARGV[0]) eq '--help' || lc($ARGV[0]) eq '-help' || lc($ARGV[0]) eq '-h' || lc($ARGV[0]) eq '--h' || lc($ARGV[0]) eq '/?') {
	print "\nUsage:\n     perl bdsest.pl PATH_TO_SCAN\n";
	print "\nExample:\n     'perl bdsest.pl src'\n     This will scan the folder \'src\' in the current directory, and then write to \n     BD_MBE_src.html into the current directory.\n\n";
	exit;
}


my @sourceextensions = ('.+\.ada$', '.+\.as$', '.+\.asm$', '.+\.asp$', '.+\.aspx$', '.+\.awk$',
			'.+\.bas$', '.+\.bms$',
			'.+\.c$', '.+\.c\+\+$', '.+\.cbl$', '.+\.cc$', '.+\.cfc$', '.+\.cfm$', '.+\.cgi$', '.+\.cls$', '.+\.cpp$', '.+\.cpy$', '.+\.cs$', '.+\.cxx$',
			'.+\.el$', '.+\.erl$',
			'.+\.f$', '.+\.f77$', '.+\.f90$', '.+\.for$', '.+\.fpp$', '.+\.frm$',
			'.+\.g77$', '.+\.g90$', '.+\.groovy',
			'.+\.h$', '.+\.hh$', '.+\.hpp$', '.+\.hrl$', '.+\.htm$', '.+\.html$', '.+\.hxx$',
			'.+\.idl$',
			'.+\.java$', '.+\.js$', '.+\.jsp$', '.+\.jws$',
			'.+\.l$', '.+\.lisp$',
			'.+\.m$','.+\.mak$',
 		    '.+\.pas$', '.+\.php$', '.+\.php3$', '.+\.php4$', '.+\.pl$', '.+\.pm$', '.+\.py$',
			'.+\.R$', '.+\.rb$', '.+\.rc$', , '.+\.rexx',
			'.+\.s$', '.+\.scm$', '.+\.sh$', '.+\.sqb$', '.+\.sql$',
			'.+\.tcl$', '.+\.tk$','.+\.txt$',
 		    '.+\.v$', '.+\.vb$', '.+\.vbs$', '.+\.vhdl$',
			'.+\.xml$',
			'.+\.y$',
			'.*copyi.*', '.*copyr.*', '.*gpl.*', '.*legal.*', '.*licen.*', '.*readme.*');
			  
my @binaryextensions = ('.+\.a$',
			'.+\.bmp$','.+\.bz2$','.+\.bzip$',
			'.+\.class$',
			'.+\.deb$','.+\.debian$','.+\.dll$',
			'.+\.ear$', '.+\.exe$',
			'.+\.gem$','.+\.gif$', '.+\.gz$',
			'.+\.jar$', '.+\.jpg$',
			'.+\.lib$',
			'.+\.o$', '.+\.obj$', '.+\.ocx$',
			'.+\.png$',
			'.+\.rar$',	'.+\.rpm$',
			'.+\.sar$','.+\.so$',
			'.+\.tar$', '.+\.tb2$', '.+\tbz$', '.+\tbz2$','.+\.tgz$', '.+\.tiff$',
			'.+\.war$',
			'.+\.zip');			  
			  

my @sourceextnames = ('*.ada', '*.as', '*.asm', '*.asp', '*.aspx', '*.awk',
			'*.bas', '*.bms',
			'*.c', '*.c++', '*.cbl', '*.cc', '*.cfc', '*.cfm', '*.cgi', '*.cls', '*.cpp', '*.cpy', '*.cs', '*.cxx',
			'*.el', '*.erl',
			'*.f', '*.f77', '*.f90', '*.for', '*.fpp', '*.frm',
			'*.g77', '*.g90', '*.groovy',
			'*.h', '*.hh', '*.hpp', '*.hrl', '*.htm', '*.html', '*.hxx',
			'*.idl',
		    '*.java', '*.js', '*.jsp', '*.jws',
			'*.l', '*.lisp',
			'*.m', '*.mak',
			'*.pas', '*.php', '*.php3', '*.php4', '*.pl', '*.pm', '*.py',
			'*.R', '*.rb', '*.rc', '*.rexx',
			'*.s', '*.scm', '*.sh', '*.sqb', '*.sql',
			'*.tcl', '*.tk', '*.txt',
			'*.v', '*.vb', '*.vbs', '*.vhdl',
			'*.xml',
			'*.y',
			'*copyi*', '*copyr*', '*gpl*', '*legal*', '*licen*', '*readme*');

my @binaryextnames = ('*.a',
			'*.bmp','*.bz2','*.bzip',
			'*.class',
			'*.deb','*.debian','*.dll',
			'*.ear', '*.exe',
			'*.gem','*.gif', '*.gz',
			'*.jar', '*.jpg',
			'*.lib',
			'*.o', '*.obj', '*.ocx',
			'*.png',
			'*.rar','*.rpm',
			'*.sar','*.so',
			'*.tar','*.tb2','*tbz','*tbz2', '*.tgz', '*.tiff',
			'*.war',
			'*.zip');

my @sourcepatterns = map { qr/$_/i } @sourceextensions;
my @binarypatterns = map { qr/$_/i } @binaryextensions;
my @plppat = map { qr/$_/i } ( '.+\.plp$' );
my $sourcepcount = $#sourcepatterns;
my $binarypcount = $#binarypatterns;

my @source_sizes = ();
my @source_num_files = ();
for my $i (0..$sourcepcount) {
	$source_sizes[$i] = 0;
	$source_num_files[$i] = 0;
}

my @binary_sizes = ();
my @binary_num_files = ();

for my $a (0..$binarypcount) {
	$binary_sizes[$a] = 0;
	$binary_num_files[$a] = 0;
}

my $source_scanned_bytes = 0;
my $source_scanned_files = 0;

my $binary_scanned_bytes = 0;
my $binary_scanned_files = 0;

my $unscanned_bytes = 0;
my $unscanned_files = 0;
my $total_bytes = 0;
my $total_files = 0;

my %unmatched_bytes = ();
my %unmatched_files = ();
$unmatched_files{'other'} = 0;
$unmatched_bytes{'other'} = 0;

sub format_number
{
	my $num = shift;
	#1 while $num =~ s/(.*)(\d)(\d\d\d)/$1$2,$3/ ;
	#This will add commas to a number, so 1000 -> 1,000
	#	uncomment if desired - otherwise it just returns itself.
	return $num;
}

sub scale_byte_count
{
	return shift(@_) / (1024*1024);
}

sub wanted
{
	my $fname = $_;

	if ($fname =~ /$plppat[0]/) {
		return;
	}
	if (-f $fname) {
		my $size = (stat($fname))[7];
		$total_bytes += $size;
		$total_files += 1;
		for my $i (0..$sourcepcount) {
			if ($fname =~ /$sourcepatterns[$i]/) {
				$source_sizes[$i] += $size;
				$source_scanned_bytes += $size;
				$source_num_files[$i] += 1;
				$source_scanned_files += 1;
				return;
			}
		}
			
		for my $i (0..$binarypcount) {
			if ($fname =~ /$binarypatterns[$i]/) {
				$binary_sizes[$i] += $size;
				$binary_scanned_bytes += $size;
				$binary_num_files[$i] += 1;
				$binary_scanned_files += 1;
				return;
			}
		}
			
		my $suffix = lc ((fileparse ($fname, ('\.[^\.]+$')))[2]);
		$unscanned_files += 1;
		$unscanned_bytes += $size;
		if ($suffix ne '') {
			$unmatched_files{$suffix} += 1;
			$unmatched_bytes{$suffix} += $size;
		}
		else {
			$unmatched_files{'other'} += 1;
			$unmatched_bytes{'other'} += $size;
		}
	}
}

my $start_run = time();

for my $i (0..$#dir_names) {
	find (\&wanted, $dir_names[$i]);
}

my $date=localtime(time);
my @weekdays = qw(Monday Tuesday Wednesday Thursday Friday Saturday Sunday);
my @months = qw(January February March April May June July August September October November December);
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
my $ampm = 'AM';
$year += 1900;
if($hour >= 12){
	if ($hour != 12) {
		$hour = $hour - 12;
	}
	$ampm = 'PM';
}
my $formattedDate = "$weekdays[$wday], $months[$mon] $mday, $year $hour:$min:$sec $ampm";

my $strConsole = '';
my $strHTML = "<?xml version=\'1.0\' encoding=\'UTF-8\'?>\n";
$strHTML .= "<!DOCTYPE html PUBLIC \'-//W3C//DTD XHTML 1.0 Transitional//EN\' \'http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\'>\n";
$strHTML .= "<html xmlns=\'http://www.w3.org/1999/xhtml\' lang=\'en\'>\n";
$strHTML .= "<head>\n<title>Black Duck Software Megabyte Estimate</title>\n<meta http-equiv=\'content-type\' content=\'text/html;charset=utf-8\'/>\n</head>\n<body>\n";

my $abs_path = $dir_names[0];

$strConsole .= "bdsest.pl v2.01 11/16/2015\n\n";
$strConsole .= "Created: $date\n\n";
$strConsole .= "Project Path: $abs_path \n\n";
$strConsole .= sprintf "Total MBytes:%9.3f\n", scale_byte_count($total_bytes) . "\n\n";
$strConsole .= "========================================\n";
$strConsole .= "     Source Files to be Scanned\n\n";
$strConsole .= "File Type              Files      MBytes\n";
$strConsole .= "========================================\n";

$strHTML .= "<p><font face=\'arial\' size=\'-1\'><b>Created: " . $formattedDate . " </b></font></p>";
$strHTML .= "<p><font face=\'arial\' size=\'-1\'><b>Project Path: $abs_path </b></font></p>\n";
$strHTML .= "<p><font face='arial' size='-1'><b>Total Mbs: " . sprintf("%9.3f\n", scale_byte_count($total_bytes)) . "</b></font></p>\n";
$strHTML .= "<p><font face='arial' size='+1'><b>Source</b></font></p>\n<table width='500' border='1' cellpadding='3' cellspacing='0'>\n<tr><td><font face='arial' size='-1'><b>Search Patterns</b></font></td></tr>\n";
$strHTML .= "<tr><td><font face='arial' size='-1'>@sourceextnames</font></td></tr>\n</table>\n";
$strHTML .= "<table width='500' border='1' cellpadding='3' cellspacing='0'>\n<tr><td align='left'><font face='arial' size='-1'><b>Found Patterns</b></font></td><td align='right'><font face='arial' size='-1'><b>Files</b></font></td><td align='right'><font face='arial' size='-1'><b>MegaBytes</b></font></td></tr>\n";

for my $i (0..$sourcepcount) {
	if ($source_num_files[$i] > 0) {
		$strConsole .= sprintf "%-16s  %9s  %11.3f\n", $sourceextnames[$i], format_number($source_num_files[$i]), scale_byte_count($source_sizes[$i]);		
		$strHTML .= "<tr><td align='left'><font face='arial' size='-1'>" . sprintf("%-16s", $sourceextnames[$i]) . "</font></td><td align='right'><font face='arial' size='-1'>" . sprintf("%9s", format_number($source_num_files[$i])) . "</font></td><td align='right'><font face='arial' size='-1'>" . sprintf("%11.3f", scale_byte_count($source_sizes[$i])) . "</font></td></tr>\n";
	}
}

$strConsole .= "----------------------------------------\n";
$strConsole .= sprintf "Totals            %9s  %11.3f\n\n",  format_number($source_scanned_files), scale_byte_count($source_scanned_bytes);
$strHTML .= "<tr><td align='left'><font face='arial' size='-1'><b>Total:</b></font></td><td align='right'><font face='arial' size='-1'><b>" . sprintf("%9s",  format_number($source_scanned_files)) . "</b></font></td><td align='right'><font face='arial' size='-1'><b>" . sprintf("%11.3f", scale_byte_count($source_scanned_bytes)) . "</b></font></td></tr>\n";
$strHTML .= "</table>\n";

$strConsole .= "     Binary Files to be Scanned\n\n";
$strConsole .= "File Type              Files      MBytes\n";
$strConsole .= "========================================\n";
$strHTML .= "<p><font face='arial' size='+1'><b>Binary</b></font></p>\n<table width='500' border='1' cellpadding='3' cellspacing='0'>\n<tr><td><font face='arial' size='-1'><b>Search Patterns</b></font></td></tr>\n";
$strHTML .= "<tr><td><font face='arial' size='-1'>@binaryextnames</font></td></tr>\n</table>\n";
$strHTML .= "<table width='500' border='1' cellpadding='3' cellspacing='0'>\n<tr><td align='left'><font face='arial' size='-1'><b>Found Patterns</b></font></td><td align='right'><font face='arial' size='-1'><b>Files</b></font></td><td align='right'><font face='arial' size='-1'><b>MegaBytes</b></font></td></tr>\n";
for my $i (0..$binarypcount) {
	if ($binary_num_files[$i] > 0) {
		$strConsole .= sprintf "%-16s  %9s  %11.3f\n", $binaryextnames[$i], format_number($binary_num_files[$i]), scale_byte_count($binary_sizes[$i]);
		$strHTML .= "<tr><td align='left'><font face='arial' size='-1'>" . sprintf("%-16s", $binaryextnames[$i]) . "</font></td><td align='right'><font face='arial' size='-1'>" . sprintf("%9s", format_number($binary_num_files[$i])) . "</font></td><td align='right'><font face='arial' size='-1'>" . sprintf("%11.3f", scale_byte_count($binary_sizes[$i])) . "</font></td></tr>\n";
	}
}
$strConsole .= "----------------------------------------\n";
$strConsole .= sprintf "Totals            %9s  %11.3f\n\n\n", format_number($binary_scanned_files), scale_byte_count($binary_scanned_bytes);
$strHTML .= "<tr><td align='left'><font face='arial' size='-1'><b>Total:</b></font></td><td align='right'><font face='arial' size='-1'><b>" . sprintf("%9s",format_number($binary_scanned_files)) . "</b></font></td><td align='right'><font face='arial' size='-1'><b>" . sprintf("%11.3f",scale_byte_count($binary_scanned_bytes)) . "</b></font></td></tr>\n";
$strHTML .= "</table>\n<p><font face='arial' size='+1'><b>Other</b></font></p>\n";
$strHTML .= "<table width='500' border='1' cellpadding='3' cellspacing='0'><tr><td align='left'><font face='arial' size='-1'><b>Unsearched Patterns</b></font></td><td align='right'><font face='arial' size='-1'><b>Files</b></font></td><td align='right'><font face='arial' size='-1'><b>MegaBytes</b></font></td></tr>\n";

$strConsole .= "       Other Files to be Scanned\n\n";
$strConsole .= "File Type              Files      MBytes\n";
$strConsole .= "========================================\n";

for my $key (keys %unmatched_files) {
	if ($key ne 'other') {
		$strConsole .= sprintf "%-16s  %9s  %11.3f\n", '*'.$key, format_number($unmatched_files{$key}), scale_byte_count($unmatched_bytes{$key});
		$strHTML .= "<tr><td align='left'><font face='arial' size='-1'>" . sprintf("%-16s", '*'.$key) . "</font></td><td align='right'><font face='arial' size='-1'>" . sprintf("%9s",format_number($unmatched_files{$key})) . "</font></td><td align='right'><font face='arial' size='-1'>" . sprintf("%11.3f", scale_byte_count($unmatched_bytes{$key})) . "</font></td></tr>\n";
	}
}
$strConsole .= sprintf "%-16s  %9s  %11.3f\n", "other", format_number($unmatched_files{'other'}), scale_byte_count($unmatched_bytes{'other'});
$strConsole .= "----------------------------------------\n";
$strConsole .= sprintf "Totals            %9s  %11.3f\n\n\n", format_number($unscanned_files), scale_byte_count($unscanned_bytes);
$strHTML .= "<tr><td align='left'><font face='arial' size='-1'><b>Total:</b></font></td><td align='right'><font face='arial' size='-1'><b>" . sprintf("%9s", format_number($unscanned_files)) . "</b></font></td><td align='right'><font face='arial' size='-1'><b>" . sprintf("%11.3f", scale_byte_count($unscanned_bytes)) . "</b></font></td></tr>\n";
$strHTML .= "</table>\n</body></html>\n";

$strConsole .= "****************************************\n";
$strConsole .= sprintf "Source Files to be scanned :%12s\n", format_number($source_scanned_files);
$strConsole .= sprintf "Binary Files to be scanned :%12s\n", format_number($binary_scanned_files);
$strConsole .= sprintf "Other Files to be scanned  :%12s\n", format_number($total_files - $source_scanned_files - $binary_scanned_files);
$strConsole .= sprintf "Total files                :%12s\n\n", format_number($total_files);

$strConsole .= sprintf "Source MBytes to be scanned:%12.3f\n", scale_byte_count($source_scanned_bytes);
$strConsole .= sprintf "Binary MBytes to be scaneed:%12.3f\n", scale_byte_count($binary_scanned_bytes);
$strConsole .= sprintf "Other MBytes to be scanned :%12.3f\n", scale_byte_count($total_bytes - $source_scanned_bytes - $binary_scanned_bytes);
$strConsole .= sprintf "Total MBytes               :%12.3f\n", scale_byte_count($total_bytes);
$strConsole .= "****************************************\n";

my $end_run = time();
my $run_time = $end_run - $start_run;
$strConsole .= "\nScript took $run_time seconds\n";

#Can uncomment the below line to also get output to the console
#print $strConsole;
my $fnHelper = $abs_path;
#if ($fnHelper eq '.') {
    #if they just passed in '.', then get the parent dir name and use that here
	$fnHelper = abs_path($fnHelper);
	#strip off the last part, up to either / or \
	my $last_slash = 0;
	if (rindex($fnHelper,'/') > -1) {
        $last_slash = rindex($fnHelper,'/');
		$fnHelper = substr($fnHelper,$last_slash + 1);
    }
	else{
        $last_slash = rindex($fnHelper,'\\');
		$fnHelper = substr($fnHelper,$last_slash + 1);		
	}
	#replace spaces with underscores
	$fnHelper =~ s/ /_/g;
#}

my $file = $fnHelper . '_CET.html';

open(HTML,'>', $file) or die "Could not open $file: $! \n";
print HTML $strHTML;
close(HTML) or die "Could not close $file: $! \n";


